
;;; $Revision: 1.27 $

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;   Arc initialization file                                        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(provide "arcinit")
(defparameter *arc-version* "1.06, rev July 2004")
(defparameter *arc-name* "Arc")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; the constant *computer-type* is set at launch for cross-platform ;;;
;;; compilations.  The function ms uses this constant.               ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defconstant 
  *computer-type* #+macintosh 'macintosh #+msdos 'windows #+unix 'unix)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; The function ms takes 3 args, and returns the first arg on a Mac,;;;
;;; the second arg on Unix/Linux, and the last arg on Windows        ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun ms (m u w)
"Function args:  (m u w)
Sets machine specific choices among m for macintosh u for unix
and w for windows."
  (case *computer-type* ('macintosh m) ('unix u) ('windows w)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;             PATH information                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun arc-working-directory ()
 (format nil "~a~a" (get-working-directory) (ms "" "/" "\\")))
(defun arc-home-path ()
 (format nil "~a~a" *home-path* (ms "" "" "\\")))
(defun directory-terminator () (ms ":" "/" "\\"))

(defun add-data-directory (name &key (relative-to 'library relset) (data t) 
  (library t libset))
"Function args: (name &key (relative-to 'library) (data t))
Adds the directory name to the list of data directories, and also to
the search path. If :relative-to is 'home, the directory is assumed to be a
subdirectory of the Arc home.  If :relative-to is 'library, it is assumed to
be a subdirectory of the library.  If :relative-to is anything else, the name
is assumed to be an absolute path.
Unless data is nil, added to the list of data pathnames."
  (let* ((relative-to 
           (cond (relset relative-to)
                 (libset (if library 'library 'home))
                 (t 'library)))
         (name (format nil "~a~a~a"
                 (case relative-to
                   (library (arc-home-path))
                   (home (arc-working-directory))
                   (t ""))
                 (string-trim (ms ":" "/" "\\") name)
                 (directory-terminator))))
  (if data (when (null (position name *data-pathnames* :test #'equal))
       (defparameter *DATA-PATHNAMES* 
                 (append *data-pathnames* (list name)))))
  (when (null (position name *load-pathname-defaults* :test #'equal))
       (defparameter *load-pathname-defaults* 
             (append *load-pathname-defaults* (list name))))))

(if (null (boundp '*home-path*)) (defparameter *home-path* ""))
(defparameter *data-pathnames* nil)

;; Basic search directories, all platforms
(add-data-directory "Extras" :library nil :data nil)  ; local Extras directory
(add-data-directory "Extras" :library t :data nil)  ; Global Extras directiory
(add-data-directory "Data" :library t)       ; library Data directory
(add-data-directory "Data" :library nil)     ; local Data directory
(add-data-directory (ms "Data:ARCG" "Data/ARCG" "Data\\ARCG"))
(add-data-directory (ms "Data:NLR" "Data/NLR" "Data\\NLR"))
(add-data-directory (ms "Data:NR-data" "Data/NR-data" "\\Data\\NR-data"))
(add-data-directory (ms "Data:ALR2" "Data/ALR2" "\\Data\\ALR2"))
(add-data-directory "Arc" :library nil :data nil)
#+unix(add-data-directory "Extras-home:" :library t :data nil)

(defun data-directories ()
"Function args: ()
Lists the data directories."
  (format t "The current data directories are~%")
  (mapcar #'(lambda (a) (format t "~a~%" a)) *DATA-PATHNAMES*)
  nil)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;   Local autoload                                            ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmacro local-autoload (name module)
  `(if (not (member ,module *modules* :test 'equal))
       (defun ,name (&rest args)
         (fmakunbound ',name)
    (require ,module)
    (eval (cons ',name (mapcar #'(lambda (x) (list 'quote x)) args))))))

(local-autoload load-arc "ldarc")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  For some reason, autoloading the compiler does not work right with the
;;;  Arc workspace.  The following override seems to solve the problem
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#-unix(add-data-directory (ms "Compiler:" "Compiler/" "\\Compiler\\") :data
nil)
#-unix(defun compile (&rest args)
        (fmakunbound 'compile)
        (require (make-pathname :name "cmpfront" 
                                :directory '(:relative "Compiler")))
        (apply #'compile args))
#-unix(defun compile-file (&rest args)
           (fmakunbound 'compile)
           (require (make-pathname :name "cmpfront" 
                                   :directory '(:relative "Compiler")))
           (apply #'compile-file args))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  copyright                                                          ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun copyright ()
"Arc, Version 1.06, March 2004
Copyright (C) 1999-2004 R. Dennis Cook and Sanford Weisberg
This program is free software; you can redistribute it and/or modify it under 
the terms of the General Public License distributed with this software in the
file license.

This program is distributed in the hope that it will be useful, but WITHOUT 
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
FOR A PARTICULAR PURPOSE.  See the the file license.txt for more details."
(format t "; ~a ~a, (C) 1999-2004, R. D. Cook & S. Weisberg.
; Type (help 'copyright) to see the license agreement.~%" 
          *arc-name* *arc-version*))

;(copyright)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                 Global settings most users won't change             ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *plotof-list* '(:variate :text)
"Variable types in the Plot of dialog")

(defparameter *text-width-min* '(2 6 6)
"Text width for very small text-items")

(defparameter *text-width-short* '(5 10 10)
"Text width for one number text-items")

(defparameter *text-width-small* *text-width-short*)

(defparameter *text-width-short2* '(12 20 20)
"Text width for one word text-items")

(defparameter *text-width-medium* '(20 30 30)
"Text width for longer text-items")

(defparameter *text-width-long* '(30 45 45)
"Text width for even longer text-items")

(defparameter *text-width-max* '(35 80 70)
"Text width for full line text-items")

(defparameter  *PURE-ERROR* T
"If t, compute and display pure error in ANOVA, when 
available.  If nil, do not display.")

(defparameter  *LATEX-PRINT-FORMAT* NIL
"If t, add LaTeX code in some tables and displays.  Default 
is nil.")

(defparameter  *JITTER-RADIUS* 0.05
"To avoid overplotting, points can be jittered by adding
a small random number before plotting.  The average size
of the random number is *jitter-radius* standard deviations.")
 
(defparameter  *PLOT-SD-SHADE* NIL
"If t, use shaded areas to show variability smooths on
scatterplots.  If nil, use two lines for boundaries.
Seems to work only on Unix, not Mac, not Windows.")
 
(defparameter  *POINT-SYMBOL* '(DISK X DIAMOND CROSS SQUARE WEDGE1)
"This gives the list of symbols used in point marking,
in the order they are used.")

(defparameter  *POINT-COLOR* '(NIL RED BLUE MAGENTA GREEN CYAN YELLOW)
"This gives the list of colors used in point marking,
in the order they are used nil means the drawing color,
which is black on a white background and white on a black
background.")
 
(defparameter  *PLOTCONTROL-MARGIN* 10
"Number of pixels from left edge of screen to the plot
controls.  Default = 10.")

(defparameter  *RELATIVE-SLIDEBAR-LENGTH* 1
"If slidebars look too long, set this to a number
smaller than 1.0.")

(defparameter  *BUTTON-SIZE* 10
"On 3D plots, the rotation buttons are 10 pixel squares.  
You can make the buttons larger by changing the 10 below to
20 or some other size.")

(defparameter  *PHANTOM-TRANSFORMATION-SLIDEBARS* T
"Transformation slidebars are shown in gray on scatterplot 
matrices if a variable has nonpositive values.  You can
skip printing these slidebars by setting this keyword to
nil.")

(defparameter  *PRINT-OFFSET* 0
"Text on graphs will be moved up by *print-offset* pixels.
This may improve printing of graphs on a Macintosh.")

(defparameter  *PALETTES-ON-BOXPLOTS* NIL
"The color and symbol palettes are generally not included 
on boxplots, but you can have them appear by setting to t.")

(defparameter  *REDEFINE-LOAD* T
"The `load' function has been changed to handle
data files correctly.  If this causes problems, 
change this keyword to nil.")

(defparameter  *SHOW-PLOT-FOR-ONE-DATASET* T
"By default, plots for only one dataset are visible on the 
screen at a time; others are hidden.  You can have all 
plots present by setting to nil.")

(defparameter  *LINK-WITHIN-DATASET* T
"Plots are linked by default only within a dataset.  To set 
linking globally, usually not a good idea, set to nil.")

(defparameter  *GREG-PREFIX* "gr"
"In graphical regression, successive linear combinations of 
predictors are saved as variables in the data set.  The 
names of these linear combinations are by default LC1, LC2, 
and so on.  You can change the prefix LC by entering the 
new value.")

(defparameter  *SET-TO-ZERO* 2.22045e-15
"Computations that should produce an answer that is exactly 
equal to zero will often give a number that is very small, 
but non-zero, due to rounding error.  To get `pretty' printing,
any computed value that is smaller than this keyword is printed 
as exactly zero (this effects only printing, not computations).")

(defparameter  *PREDICTION-DIALOG* T
"If t, creates a menu item in regression menus for predictions, fitted values
and contrasts.  If a settings file is saved, this change takes effect the next
time the program is launched.")

(defparameter *confidence-curve-max-coverage* .96
"A number between 0 and 1, typically between .95 and .999, specifying how far
into the extreme tail computations should be done for confidence curves.  The
default of .96 seems a good choice, larger values large .99 lead to erratic
behavior.")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;  Adds gray background graphs to the options menu of all graphs
;;;  Also, alls some plot controls to be drawn dim
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;(make-color 'grayback .75 .75 .75)
(defparameter *shade-colors* nil)
(defparameter *grayback* nil)

(defun grayback ()
 (unless *grayback*
     (make-color 'grayback .75 .75 .75)
     (defparameter *grayback* 'grayback))
 'grayback)

(defun dim-color () (grayback))
     

(defun shade-colors ()
 (unless *shade-colors*
     (make-color 'shade0 .75 .75 .75)  ; shading for white
     (make-color 'shade1 .75 .75 .75)  ; shading for black
     (make-color 'shade2 .75 .50 .50)  ; shading for red
     (make-color 'shade3 .50 .75 .50)  ; shading for green
     (make-color 'shade4 .50 .50 .75)  ; shading for blue
     (make-color 'shade5 .50 .75 .75)  ; shading for cyan
     (make-color 'shade6 .75 .50 .75)  ; shading for magenta
     (make-color 'shade7 .75 .75 .50)  ; shading for yellow
     (defparameter *shade-colors* (list 'shade0 'shade1 'shade2 'shade3 'shade4
                                  'shade5 'shade6 'shade7))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;              More global parameters set at runtime                  ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(send histogram-proto :size 250 175)
(defconstant *c-double-fix* (if (> xls-minor-release 46) 'c-double nil))
(defparameter *arc-packages* nil)
(defparameter *float-format* "%g")
(defparameter *title-on-plot-menu-item* t)
(defparameter *close-plot-menu-item* nil)
(defparameter *extract-horizontal-only* nil)
(defparameter *parametric-smoothers*
 '(ols-smooth huber-m-smooth logistic-smooth poisson-smooth 
   gamma-smooth power-curve))
(defparameter *nonparametric-smoothers*
  '(lowess-smooth mean-sd-smooth slice-smooth ))
(defparameter *density-smoothers*
  '(gaussian-density )) ; biweight-density deleted
(defparameter *3d-parametric-smoothers*
  '(3d-ols-proto 3d-huber-m-smooth 3d-logistic-smooth 
    3d-poisson-smooth 3d-gamma-smooth))
(defparameter *linear-plotlist*
  '(:fit-values :Residuals :Outlier-T :Leverages :Cooks-D))
(defparameter *nonlinear-plotlist*
  '(:fit-values :Residuals :Ext-Stud-Res :Proj-Res :Jac-Lev :Leverages
    :Cooks-D))
(defparameter *glm-plotlist*
  '(:Eta\'u :Fit-Means :Chi-Residuals :Dev-Residuals :T-Residuals
    :Leverages :Cooks-D))
(defparameter *binomial-plotlist*
  '(:Eta\'u :Obs-Fraction :Fit-Fraction :Chi-Residuals :Dev-Residuals
    :T-Residuals :Leverages :Cooks-Distances))
(defparameter *power-transformation-selections*
  '(-2 -1.5 -1 -0.67 -0.5 -0.33 -0.25 -0.1 0 0.1 0.25 0.33 0.5 0.67 1 1.5 2))
(defparameter *transform-slidebar-choices*
  '(-1 -0.67 -0.5 -0.33 -0.25 0 0.25 0.33 0.5 0.67 1 1.25 1.5 1.75 2))
(defparameter *power-curve-choices*
  '(-2 -1.5 -1 -0.67 -0.5 -0.33 -0.25 0 0.25 0.33 0.5 0.67 1 1.25 1.5 1.75 2))
(defparameter *smoother-bandwidth-choices*
  '(0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1))
(defparameter *kernel-density-choices*
  '(0.3 0.5 0.6 0.8 0.9 1 1.1 1.2))  ; changed 1/22/1998
(defparameter *histogram-numbins*   ; for histogram number-of-bins slide bar
     '(2 3 4 5 6 7 8 9 10 12 14 16 18 20 22 24 26 28 30))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;   To allow Russ Lenth's Excel interface to work on WINDOWS       ;;;
;;;   Also, set the graphics font size for windows to 14 pts         ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

#+msdos (set-macro-character #\[ #'(lambda (x y)))
#+msdos (set-macro-character #\] #'(lambda (x y)))
;#+msdos (def *graphics-font-size* 14)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;   To allow automatic loading of the contents of a directory      ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun load-files-in-directory (&optional (dir "Extras")
                                          (path (get-working-directory)))
"Function args: ()
This function looks in the subdirectory Extras of the *home-path*, and then
loads all files in that subdirectory."
   (let* ((ans (files-in-directory :path path :dir dir)))
    (when ans (apply #'lfid1 ans))))

(defun lfid1 (&key path files)
     (mapcar #'require files)
     (mapcar #'provide files))

(defun display-file-names (&rest args)
"Function args:  (&key (path (get-working-directory)) dir)
Returns the names of all the files ending in .lsp, .fsl, or .txt 
in the directory specified."
  (let* ((ans (apply #'files-in-directory :fsl-files nil :txt-files nil args))
         (ansf (apply #'files-in-directory :lsp-files nil :txt-files nil args))
         (anst (apply #'files-in-directory :lsp-files nil :fsl-files nil
                             :txt-files t args)))
   (format t "Path = ~a~%" (select ans 1))
   (when ans (mapcar #'(lambda (a) (format t "~a.lsp~%" a)) (select ans 3)))
   (when ansf (mapcar #'(lambda (a) (format t "~a.fsl~%" a)) (select ansf 3)))
   (when anst (mapcar #'(lambda (a) (format t "~a.txt~%" a)) (select anst 3)))
   nil))


(defun files-in-directory (&key (path (get-working-directory))
                                (lsp-files t) (fsl-files t) (txt-files nil)
                                (dir "Extras"))
"Function args: (&key path dir)
Returns the names of all files ending with .lsp or .fsl, with the directory 
and suffix stripped off."
   (let* ((d (format nil "~a~a~a~a" path (ms "" "/" "\\") dir
                                          (ms ":" "/" "\\")))
;  (let* ((d (format nil "~a~a~a" path (ms "" "/" "\\") dir ))
          (files (append (if lsp-files (directory (format nil "~a*.lsp" d)))
                         (if fsl-files (directory (format nil "~a*.fsl" d)))
                         (if txt-files (directory (format nil "~a*.txt" d)))
                         (if lsp-files (directory (format nil "~a*.LSP" d)))
                         (if txt-files (directory (format nil "~a*.TXT" d)))
                         (if fsl-files (directory (format nil "~a*.FSL" d)))))
          (f1 (when files (mapcar #'(lambda (f) 
                             (select f (iseq (length d) (- (length f) 5))))
                             files))))
    (when f1
     (list :path d 
           :files (qsort (remove-duplicates f1 :test #'equal))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;             Load .fsl files if no workspace                         ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(cond ((boundp 'arc)                    ;; if Arc workspace
#+unix (setf xlisp::*fake-menu-bar* nil)  ;;  delete the Unix menu bar
       (send arc :slot-value 'menu nil) ;;  delete the old Arc menu
       (unless *ignore-extras*
               (load-files-in-directory "Extras")
               (load-files-in-directory "Extras" *home-path*)
               )
       (when (probe-file "settings.lsp")
             (load "settings"))           ;;  load settings, if it exists
       (send arc :menu))                ;;  restore the menu and menubar
      (t (load-arc)))                   ;; else no Arc workspace
;;
;; make sure the fake menu bar on unix cannot be dismissed
;;
#+unix (defmeth xlisp::*fake-menu-bar* :close ())
;; the following should be inside the above cond statement.
       (when *redefine-load*              ;;  redefine load function
             (defun load (&rest f) (apply #'arc-load f)))
         
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;             Display the copyright statement                         ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(if (boundp '*auto-save*)
    (auto-save *auto-save*)) ;;; turns on auto saving if the keyword is not nil
(copyright)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;             End of file                                             ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
